import sdl2
import sdl2.ext

from aerforgesdl.color import Color
from aerforgesdl.vec2 import Vec2

class Entity:
    def __init__(self, window, width = 20, height = 20, x = 0, y = 0, color = Color(240, 240, 240), fill = True, scripts = [], add_to_objects = True):
        self.window = window

        self.width = width
        self.height = height

        self.x = x
        self.y = y

        self.color = color
        self.fill = fill

        self.scripts = scripts

        self.destroyed = False

        self.add_to_objects = add_to_objects

        if self.add_to_objects:
            self.window.objects.append(self)

        self.rect = sdl2.SDL_FRect(self.x, self.y, self.width, self.height)

    def _update(self):
        for script in self.scripts:
            script.update(self)

    def draw(self):
        if not self.destroyed:
            sdl2.SDL_SetRenderDrawColor(self.window.renderer, self.color.r, self.color.g, self.color.b, self.color.a)

            self.rect = sdl2.SDL_FRect(self.x, self.y, self.width, self.height)

            sdl2.SDL_RenderDrawRectF(self.window.renderer, self.rect)

            if self.fill:
                sdl2.SDL_RenderFillRectF(self.window.renderer, self.rect)

            self.rect = sdl2.SDL_FRect(self.x, self.y, self.width, self.height)

    def set_color(self, color):
        self.color = color

    def get_alpha(self):
        return self.color

    def get_alpha(self):
        return self.color.a

    def set_alpha(self, alpha):
        if alpha > 0:
            self.alpha = alpha

        else:
            self.alpha = 0

        self.color.a = int(self.alpha)
   
    def get_x(self):
        return self.x

    def get_y(self):
        return self.y

    def set_x(self, x):
        self.x = x

    def set_y(self, y):
        self.y = y

    def get_width(self):
        return self.width

    def get_height(self):
        return self.height

    def set_width(self, width):
        self.width = width

    def set_height(self, height):
        self.height = height

    def center(self):
        if not self.destroyed:
            self.x = self.window.width / 2 - self.width / 2
            self.y = self.window.height / 2 - self.height / 2

    def center_x(self):
        if not self.destroyed:
            self.x = self.window.width / 2 - self.width / 2

    def center_y(self):
        if not self.destroyed:
            self.y = self.window.height / 2 - self.height / 2

    def hit(self, entity):
        if not self.destroyed:
            if isinstance(entity, Vec2):
                left, top, right, bottom = self.rect.x, self.rect.y, self.rect.x + self.rect.w, self.rect.y + self.rect.h
                bleft, btop, bright, bbottom = entity.x, entity.y, entity.x, entity.y

            elif isinstance(entity, tuple):
                left, top, right, bottom = self.rect.x, self.rect.y, self.rect.x + self.rect.w, self.rect.y + self.rect.h
                bleft, btop, bright, bbottom = entity[0], entity[1], entity[0], entity[1]

            else:
                left, top, right, bottom = self.rect.x, self.rect.y, self.rect.x + self.rect.w, self.rect.y + self.rect.h
                bleft, btop, bright, bbottom = entity.rect.x, entity.rect.y, entity.rect.x + entity.rect.w, entity.rect.y + entity.rect.h
                
            return (bleft < right and bright > left and btop < bottom and bbottom > top)

        else:
            return False

    def hit2(self, entity, collision_tolreance = 10):
        if self.hit(entity):
            left, top, right, bottom = self.rect.x, self.rect.y, self.rect.x + self.rect.w, self.rect.y + self.rect.h
            bleft, btop, bright, bbottom = entity.rect.x, entity.rect.y, entity.rect.x + entity.rect.w, entity.rect.y + entity.rect.h

            if abs(btop - bottom) < collision_tolreance:
                return "bottom"

            elif abs(bbottom - top) < collision_tolreance:
                return "top"

            elif abs(bright - left) < collision_tolreance:
                return "left"

            elif abs(bleft - right) < collision_tolreance:
                return "right"

            else:
                return "unknown"
        
        else:
            return ""

    def destroy(self):
        self.destroyed = True

        if self.add_to_objects:
            try:
                self.window.objects.pop(self.window.objects.index(self))

            except:
                pass

    def add_script(self, script):
        self.scripts.append(script)

    def remove_script(self, script):
        self.scripts.pop(self.scripts.index(script))